@@ -49,7 +49,7 @@ impl Default for NewPluginScreen {
             entering_config_val: false,
             selected_config_index: None,
             request_ids: vec![],
-            load_in_background: true,
+            load_in_background: false,
             colors: Palette::default(),
         }
     }
@@ -99,7 +99,7 @@ impl NewPluginScreen {
         };
         print_text_with_coordinates(url_field, 0, 2, None, None);
         let url_helper =
-            NestedListItem::new(format!("<Ctrl d> - Load from Disk")).color_range(3, ..=8);
+            NestedListItem::new(format!("<Ctrl f> - Load from Disk")).color_range(3, ..=8);
         print_nested_list_with_coordinates(vec![url_helper], 0, 3, None, None);
     }
     fn render_configuration_title(&self) {
@@ -353,7 +353,7 @@ impl NewPluginScreen {
         let (mut should_render, mut should_close) = (false, false);
 
         match key.bare_key {
-            BareKey::Char(character) if key.has_no_modifiers() && character != ' ' => {
+            BareKey::Char(character) if key.has_no_modifiers() => {
                 if let Some(field) = self.get_field_being_edited_mut() {
                     field.push(character);
                 }
@@ -457,7 +457,7 @@ impl NewPluginScreen {
                     should_render = true;
                 }
             },
-            BareKey::Char('d') if key.has_modifiers(&[KeyModifier::Ctrl]) => {
+            BareKey::Char('f') if key.has_modifiers(&[KeyModifier::Ctrl]) => {
                 let mut args = BTreeMap::new();
                 let request_id = Uuid::new_v4();
                 self.request_ids.push(request_id.to_string());
@@ -965,7 +965,7 @@ impl State {
     pub fn handle_main_screen_key(&mut self, key: KeyWithModifier) -> bool {
         let mut should_render = false;
         match key.bare_key {
-            BareKey::Char(character) if key.has_no_modifiers() && character != ' ' => {
+            BareKey::Char(character) if key.has_no_modifiers() => {
                 self.search_term.push(character);
                 self.update_search_term();
                 self.reset_selection();
