@@ -5,7 +5,7 @@ use crate::{
 };
 use zellij_utils::{
     channels::{Receiver, SenderWithContext, OPENCALLS},
-    data::{InputMode, Key},
+    data::{InputMode, KeyWithModifier},
     errors::{ContextType, ErrorContext, FatalError},
     input::{
         actions::Action,
@@ -96,7 +96,7 @@ impl InputHandler {
                                 &raw_bytes,
                                 Some((&self.config.keybinds, &self.mode)),
                             );
-                            self.handle_key(&key, raw_bytes);
+                            self.handle_key(&key, raw_bytes, false);
                         },
                         InputEvent::Mouse(mouse_event) => {
                             let mouse_event =
@@ -106,15 +106,15 @@ impl InputHandler {
                         InputEvent::Paste(pasted_text) => {
                             if self.mode == InputMode::Normal || self.mode == InputMode::Locked {
                                 self.dispatch_action(
-                                    Action::Write(bracketed_paste_start.clone()),
+                                    Action::Write(None, bracketed_paste_start.clone(), false),
                                     None,
                                 );
                                 self.dispatch_action(
-                                    Action::Write(pasted_text.as_bytes().to_vec()),
+                                    Action::Write(None, pasted_text.as_bytes().to_vec(), false),
                                     None,
                                 );
                                 self.dispatch_action(
-                                    Action::Write(bracketed_paste_end.clone()),
+                                    Action::Write(None, bracketed_paste_end.clone(), false),
                                     None,
                                 );
                             }
@@ -140,8 +140,11 @@ impl InputHandler {
                         _ => {},
                     }
                 },
-                Ok((InputInstruction::SwitchToMode(input_mode), _error_context)) => {
-                    self.mode = input_mode;
+                Ok((
+                    InputInstruction::KeyWithModifierEvent(key_with_modifier, raw_bytes),
+                    _error_context,
+                )) => {
+                    self.handle_key(&key_with_modifier, raw_bytes, true);
                 },
                 Ok((
                     InputInstruction::AnsiStdinInstructions(ansi_stdin_instructions),
@@ -168,16 +171,19 @@ impl InputHandler {
             }
         }
     }
-    fn handle_key(&mut self, key: &Key, raw_bytes: Vec<u8>) {
-        let keybinds = &self.config.keybinds;
-        for action in
-            keybinds.get_actions_for_key_in_mode_or_default_action(&self.mode, key, raw_bytes)
-        {
-            let should_exit = self.dispatch_action(action, None);
-            if should_exit {
-                self.should_exit = true;
-            }
-        }
+    fn handle_key(
+        &mut self,
+        key: &KeyWithModifier,
+        raw_bytes: Vec<u8>,
+        is_kitty_keyboard_protocol: bool,
+    ) {
+        // we interpret the keys into actions on the server side so that we can change the
+        // keybinds at runtime
+        self.os_input.send_to_server(ClientToServerMsg::Key(
+            key.clone(),
+            raw_bytes,
+            is_kitty_keyboard_protocol,
+        ));
     }
     fn handle_stdin_ansi_instruction(&mut self, ansi_stdin_instructions: AnsiStdinInstruction) {
         match ansi_stdin_instructions {
@@ -302,14 +308,8 @@ impl InputHandler {
                 self.exit(ExitReason::NormalDetached);
                 should_break = true;
             },
-            Action::SwitchToMode(mode) => {
-                // this is an optimistic update, we should get a SwitchMode instruction from the
-                // server later that atomically changes the mode as well
-                self.mode = mode;
-                self.os_input
-                    .send_to_server(ClientToServerMsg::Action(action, None, None));
-            },
             Action::CloseFocus
+            | Action::SwitchToMode(..)
             | Action::ClearScreen
             | Action::NewPane(..)
             | Action::Run(_)
