@@ -1,11 +1,7 @@
-use std::{
-    collections::VecDeque,
-    io::{Read, Seek, Write},
-};
+use std::{collections::VecDeque, io::Write};
 
 use crate::plugins::PluginId;
 use log::{debug, error};
-use wasmer_wasi::{WasiFile, WasiFsError};
 use zellij_utils::{errors::prelude::*, serde};
 
 use chrono::prelude::*;
@@ -41,15 +37,6 @@ impl LoggingPipe {
     }
 }
 
-impl Read for LoggingPipe {
-    fn read(&mut self, _: &mut [u8]) -> std::io::Result<usize> {
-        Err(std::io::Error::new(
-            std::io::ErrorKind::Other,
-            "Can not reed from a LoggingPipe",
-        ))
-    }
-}
-
 impl Write for LoggingPipe {
     fn write(&mut self, buf: &[u8]) -> std::io::Result<usize> {
         if self.buffer.len() + buf.len() > ZELLIJ_MAX_PIPE_BUFFER_SIZE {
@@ -106,40 +93,6 @@ impl Write for LoggingPipe {
     }
 }
 
-impl Seek for LoggingPipe {
-    fn seek(&mut self, _pos: std::io::SeekFrom) -> std::io::Result<u64> {
-        Err(std::io::Error::new(
-            std::io::ErrorKind::Other,
-            "can not seek in a pipe",
-        ))
-    }
-}
-
-impl WasiFile for LoggingPipe {
-    fn last_accessed(&self) -> u64 {
-        0
-    }
-    fn last_modified(&self) -> u64 {
-        0
-    }
-    fn created_time(&self) -> u64 {
-        0
-    }
-    fn size(&self) -> u64 {
-        self.buffer.len() as u64
-    }
-    fn set_len(&mut self, len: u64) -> Result<(), WasiFsError> {
-        self.buffer.resize(len as usize, 0);
-        Ok(())
-    }
-    fn unlink(&mut self) -> Result<(), WasiFsError> {
-        Ok(())
-    }
-    fn bytes_available(&self) -> Result<usize, WasiFsError> {
-        Ok(self.buffer.len())
-    }
-}
-
 // Unit tests
 #[cfg(test)]
 mod logging_pipe_test {
