@@ -619,10 +619,10 @@ impl Display for CharacterStyles {
             if let Some(ansi_code) = self.underline_color {
                 match ansi_code {
                     AnsiCode::RgbCode((r, g, b)) => {
-                        write!(f, "\u{1b}[58;2;{};{};{}m", r, g, b)?;
+                        write!(f, "\u{1b}[58:2::{}:{}:{}m", r, g, b)?;
                     },
                     AnsiCode::ColorIndex(color_index) => {
-                        write!(f, "\u{1b}[58;5;{}m", color_index)?;
+                        write!(f, "\u{1b}[58:5:{}m", color_index)?;
                     },
                     AnsiCode::Reset => {
                         write!(f, "\u{1b}[59m")?;
