@@ -16,7 +16,10 @@ use std::time::{self, Instant};
 use zellij_utils::input::command::RunCommand;
 use zellij_utils::pane_size::Offset;
 use zellij_utils::{
-    data::{InputMode, Palette, PaletteColor, PaneId as ZellijUtilsPaneId, Style},
+    data::{
+        BareKey, InputMode, KeyWithModifier, Palette, PaletteColor, PaneId as ZellijUtilsPaneId,
+        Style,
+    },
     errors::prelude::*,
     input::layout::Run,
     pane_size::PaneGeom,
@@ -37,8 +40,8 @@ const UP_ARROW: &[u8] = &[27, 91, 65];
 const DOWN_ARROW: &[u8] = &[27, 91, 66];
 const HOME_KEY: &[u8] = &[27, 91, 72];
 const END_KEY: &[u8] = &[27, 91, 70];
-const BRACKETED_PASTE_BEGIN: &[u8] = &[27, 91, 50, 48, 48, 126];
-const BRACKETED_PASTE_END: &[u8] = &[27, 91, 50, 48, 49, 126];
+pub const BRACKETED_PASTE_BEGIN: &[u8] = &[27, 91, 50, 48, 48, 126];
+pub const BRACKETED_PASTE_END: &[u8] = &[27, 91, 50, 48, 49, 126];
 const ENTER_NEWLINE: &[u8] = &[10];
 const ESC: &[u8] = &[27];
 const ENTER_CARRIAGE_RETURN: &[u8] = &[13];
@@ -95,6 +98,15 @@ impl From<ZellijUtilsPaneId> for PaneId {
     }
 }
 
+impl Into<ZellijUtilsPaneId> for PaneId {
+    fn into(self) -> ZellijUtilsPaneId {
+        match self {
+            PaneId::Terminal(id) => ZellijUtilsPaneId::Terminal(id),
+            PaneId::Plugin(id) => ZellijUtilsPaneId::Plugin(id),
+        }
+    }
+}
+
 type IsFirstRun = bool;
 
 // FIXME: This should hold an os_api handle so that terminal panes can set their own size via FD in
@@ -190,92 +202,74 @@ impl Pane for TerminalPane {
             .cursor_coordinates()
             .map(|(x, y)| (x + left, y + top))
     }
-    fn adjust_input_to_terminal(&mut self, input_bytes: Vec<u8>) -> Option<AdjustedInput> {
+    fn is_mid_frame(&self) -> bool {
+        self.grid.is_mid_frame()
+    }
+    fn adjust_input_to_terminal(
+        &mut self,
+        key_with_modifier: &Option<KeyWithModifier>,
+        raw_input_bytes: Vec<u8>,
+        raw_input_bytes_are_kitty: bool,
+    ) -> Option<AdjustedInput> {
         // there are some cases in which the terminal state means that input sent to it
         // needs to be adjusted.
         // here we match against those cases - if need be, we adjust the input and if not
         // we send back the original input
-        if let Some((_exit_status, _is_first_run, run_command)) = &self.is_held {
-            match input_bytes.as_slice() {
-                ENTER_CARRIAGE_RETURN | ENTER_NEWLINE | SPACE => {
-                    let run_command = run_command.clone();
-                    self.is_held = None;
-                    self.grid.reset_terminal_state();
-                    self.set_should_render(true);
-                    self.remove_banner();
-                    Some(AdjustedInput::ReRunCommandInThisPane(run_command))
-                },
-                ESC => {
-                    // Drop to shell in the same working directory as the command was run
-                    let working_dir = run_command.cwd.clone();
-                    self.is_held = None;
-                    self.grid.reset_terminal_state();
-                    self.set_should_render(true);
-                    self.remove_banner();
-                    Some(AdjustedInput::DropToShellInThisPane { working_dir })
-                },
-                CTRL_C => Some(AdjustedInput::CloseThisPane),
-                _ => None,
-            }
-        } else {
-            if self.grid.new_line_mode {
-                if let &[13] = input_bytes.as_slice() {
-                    // LNM - carriage return is followed by linefeed
-                    return Some(AdjustedInput::WriteBytesToTerminal(
-                        "\u{0d}\u{0a}".as_bytes().to_vec(),
-                    ));
-                };
-            }
-            if self.grid.cursor_key_mode {
-                match input_bytes.as_slice() {
-                    LEFT_ARROW => {
-                        return Some(AdjustedInput::WriteBytesToTerminal(
-                            AnsiEncoding::Left.as_vec_bytes(),
-                        ));
-                    },
-                    RIGHT_ARROW => {
-                        return Some(AdjustedInput::WriteBytesToTerminal(
-                            AnsiEncoding::Right.as_vec_bytes(),
-                        ));
-                    },
-                    UP_ARROW => {
-                        return Some(AdjustedInput::WriteBytesToTerminal(
-                            AnsiEncoding::Up.as_vec_bytes(),
-                        ));
-                    },
-                    DOWN_ARROW => {
-                        return Some(AdjustedInput::WriteBytesToTerminal(
-                            AnsiEncoding::Down.as_vec_bytes(),
-                        ));
-                    },
 
-                    HOME_KEY => {
-                        return Some(AdjustedInput::WriteBytesToTerminal(
-                            AnsiEncoding::Home.as_vec_bytes(),
-                        ));
-                    },
-                    END_KEY => {
-                        return Some(AdjustedInput::WriteBytesToTerminal(
-                            AnsiEncoding::End.as_vec_bytes(),
-                        ));
-                    },
-                    _ => {},
-                };
+        if !self.grid.bracketed_paste_mode {
+            // Zellij itself operates in bracketed paste mode, so the terminal sends these
+            // instructions (bracketed paste start and bracketed paste end respectively)
+            // when pasting input. We only need to make sure not to send them to terminal
+            // panes who do not work in this mode
+            match raw_input_bytes.as_slice() {
+                BRACKETED_PASTE_BEGIN | BRACKETED_PASTE_END => {
+                    return Some(AdjustedInput::WriteBytesToTerminal(vec![]))
+                },
+                _ => {},
             }
+        }
 
-            if !self.grid.bracketed_paste_mode {
-                // Zellij itself operates in bracketed paste mode, so the terminal sends these
-                // instructions (bracketed paste start and bracketed paste end respectively)
-                // when pasting input. We only need to make sure not to send them to terminal
-                // panes who do not work in this mode
-                match input_bytes.as_slice() {
-                    BRACKETED_PASTE_BEGIN | BRACKETED_PASTE_END => {
-                        return Some(AdjustedInput::WriteBytesToTerminal(vec![]))
-                    },
-                    _ => {},
+        if self.is_held.is_some() {
+            if key_with_modifier
+                .as_ref()
+                .map(|k| k.is_key_without_modifier(BareKey::Enter))
+                .unwrap_or(false)
+            {
+                self.handle_held_run()
+            } else if key_with_modifier
+                .as_ref()
+                .map(|k| k.is_key_without_modifier(BareKey::Esc))
+                .unwrap_or(false)
+            {
+                self.handle_held_drop_to_shell()
+            } else if key_with_modifier
+                .as_ref()
+                .map(|k| k.is_key_with_ctrl_modifier(BareKey::Char('c')))
+                .unwrap_or(false)
+            {
+                Some(AdjustedInput::CloseThisPane)
+            } else {
+                match raw_input_bytes.as_slice() {
+                    ENTER_CARRIAGE_RETURN | ENTER_NEWLINE | SPACE => self.handle_held_run(),
+                    ESC => self.handle_held_drop_to_shell(),
+                    CTRL_C => Some(AdjustedInput::CloseThisPane),
+                    _ => None,
                 }
             }
-            Some(AdjustedInput::WriteBytesToTerminal(input_bytes))
+        } else {
+            if self.grid.supports_kitty_keyboard_protocol {
+                self.adjust_input_to_terminal_with_kitty_keyboard_protocol(
+                    key_with_modifier,
+                    raw_input_bytes,
+                    raw_input_bytes_are_kitty,
+                )
+            } else {
+                self.adjust_input_to_terminal_without_kitty_keyboard_protocol(
+                    key_with_modifier,
+                    raw_input_bytes,
+                    raw_input_bytes_are_kitty,
+                )
+            }
         }
     }
     fn position_and_size(&self) -> PaneGeom {
@@ -521,7 +515,7 @@ impl Pane for TerminalPane {
         self.geom.y -= count;
         self.reflow_lines();
     }
-    fn dump_screen(&mut self, _client_id: ClientId, full: bool) -> String {
+    fn dump_screen(&self, full: bool) -> String {
         self.grid.dump_screen(full)
     }
     fn clear_screen(&mut self) {
@@ -784,6 +778,34 @@ impl Pane for TerminalPane {
     fn serialize(&self, scrollback_lines_to_serialize: Option<usize>) -> Option<String> {
         self.grid.serialize(scrollback_lines_to_serialize)
     }
+    fn rerun(&mut self) -> Option<RunCommand> {
+        // if this is a command pane that has exited or is waiting to be rerun, will return its
+        // RunCommand, otherwise it is safe to assume this is not the right sort of pane or that it
+        // is not in the right sort of state
+        self.is_held.take().map(|(_, _, run_command)| {
+            self.is_held = None;
+            self.grid.reset_terminal_state();
+            self.set_should_render(true);
+            self.remove_banner();
+            run_command.clone()
+        })
+    }
+    fn update_theme(&mut self, theme: Palette) {
+        self.style.colors = theme.clone();
+        self.grid.update_theme(theme);
+        if self.banner.is_some() {
+            // we do this so that the banner will be updated with the new theme colors
+            self.render_first_run_banner();
+        }
+    }
+    fn update_arrow_fonts(&mut self, should_support_arrow_fonts: bool) {
+        self.arrow_fonts = should_support_arrow_fonts;
+        self.grid.update_arrow_fonts(should_support_arrow_fonts);
+    }
+    fn update_rounded_corners(&mut self, rounded_corners: bool) {
+        self.style.rounded_corners = rounded_corners;
+        self.frame.clear();
+    }
 }
 
 impl TerminalPane {
@@ -804,6 +826,7 @@ impl TerminalPane {
         debug: bool,
         arrow_fonts: bool,
         styled_underlines: bool,
+        explicitly_disable_keyboard_protocol: bool,
     ) -> TerminalPane {
         let initial_pane_title =
             initial_pane_title.unwrap_or_else(|| format!("Pane #{}", pane_index));
@@ -819,6 +842,7 @@ impl TerminalPane {
             debug,
             arrow_fonts,
             styled_underlines,
+            explicitly_disable_keyboard_protocol,
         );
         TerminalPane {
             frame: HashMap::new(),
@@ -910,6 +934,131 @@ impl TerminalPane {
             self.banner = None;
         }
     }
+    fn adjust_input_to_terminal_with_kitty_keyboard_protocol(
+        &self,
+        key: &Option<KeyWithModifier>,
+        raw_input_bytes: Vec<u8>,
+        raw_input_bytes_are_kitty: bool,
+    ) -> Option<AdjustedInput> {
+        if raw_input_bytes_are_kitty || key.is_none() {
+            Some(AdjustedInput::WriteBytesToTerminal(raw_input_bytes))
+        } else {
+            // here what happens is that the host terminal is operating in non "kitty keys" mode, but
+            // this terminal pane *is* operating in "kitty keys" mode - so we need to serialize the "non kitty"
+            // key to a "kitty key"
+            key.as_ref()
+                .and_then(|k| k.serialize_kitty())
+                .map(|s| AdjustedInput::WriteBytesToTerminal(s.as_bytes().to_vec()))
+        }
+    }
+    fn adjust_input_to_terminal_without_kitty_keyboard_protocol(
+        &self,
+        key: &Option<KeyWithModifier>,
+        raw_input_bytes: Vec<u8>,
+        raw_input_bytes_are_kitty: bool,
+    ) -> Option<AdjustedInput> {
+        if self.grid.new_line_mode {
+            let key_is_enter = raw_input_bytes.as_slice() == &[13]
+                || key
+                    .as_ref()
+                    .map(|k| k.is_key_without_modifier(BareKey::Enter))
+                    .unwrap_or(false);
+            if key_is_enter {
+                // LNM - carriage return is followed by linefeed
+                return Some(AdjustedInput::WriteBytesToTerminal(
+                    "\u{0d}\u{0a}".as_bytes().to_vec(),
+                ));
+            };
+        }
+        if self.grid.cursor_key_mode {
+            let key_is_left_arrow = raw_input_bytes.as_slice() == LEFT_ARROW
+                || key
+                    .as_ref()
+                    .map(|k| k.is_key_without_modifier(BareKey::Left))
+                    .unwrap_or(false);
+            let key_is_right_arrow = raw_input_bytes.as_slice() == RIGHT_ARROW
+                || key
+                    .as_ref()
+                    .map(|k| k.is_key_without_modifier(BareKey::Right))
+                    .unwrap_or(false);
+            let key_is_up_arrow = raw_input_bytes.as_slice() == UP_ARROW
+                || key
+                    .as_ref()
+                    .map(|k| k.is_key_without_modifier(BareKey::Up))
+                    .unwrap_or(false);
+            let key_is_down_arrow = raw_input_bytes.as_slice() == DOWN_ARROW
+                || key
+                    .as_ref()
+                    .map(|k| k.is_key_without_modifier(BareKey::Down))
+                    .unwrap_or(false);
+            let key_is_home_key = raw_input_bytes.as_slice() == HOME_KEY
+                || key
+                    .as_ref()
+                    .map(|k| k.is_key_without_modifier(BareKey::Home))
+                    .unwrap_or(false);
+            let key_is_end_key = raw_input_bytes.as_slice() == END_KEY
+                || key
+                    .as_ref()
+                    .map(|k| k.is_key_without_modifier(BareKey::End))
+                    .unwrap_or(false);
+            if key_is_left_arrow {
+                return Some(AdjustedInput::WriteBytesToTerminal(
+                    AnsiEncoding::Left.as_vec_bytes(),
+                ));
+            } else if key_is_right_arrow {
+                return Some(AdjustedInput::WriteBytesToTerminal(
+                    AnsiEncoding::Right.as_vec_bytes(),
+                ));
+            } else if key_is_up_arrow {
+                return Some(AdjustedInput::WriteBytesToTerminal(
+                    AnsiEncoding::Up.as_vec_bytes(),
+                ));
+            } else if key_is_down_arrow {
+                return Some(AdjustedInput::WriteBytesToTerminal(
+                    AnsiEncoding::Down.as_vec_bytes(),
+                ));
+            } else if key_is_home_key {
+                return Some(AdjustedInput::WriteBytesToTerminal(
+                    AnsiEncoding::Home.as_vec_bytes(),
+                ));
+            } else if key_is_end_key {
+                return Some(AdjustedInput::WriteBytesToTerminal(
+                    AnsiEncoding::End.as_vec_bytes(),
+                ));
+            }
+        }
+        if raw_input_bytes_are_kitty {
+            // here what happens is that the host terminal is operating in "kitty keys" mode, but
+            // this terminal pane is not - so we need to serialize the kitty key to "non kitty" if
+            // possible - if not possible (eg. with multiple modifiers), we'll return a None here
+            // and write nothing to the terminal pane
+            key.as_ref()
+                .and_then(|k| k.serialize_non_kitty())
+                .map(|s| AdjustedInput::WriteBytesToTerminal(s.as_bytes().to_vec()))
+        } else {
+            Some(AdjustedInput::WriteBytesToTerminal(raw_input_bytes))
+        }
+    }
+    fn handle_held_run(&mut self) -> Option<AdjustedInput> {
+        self.is_held.take().map(|(_, _, run_command)| {
+            self.is_held = None;
+            self.grid.reset_terminal_state();
+            self.set_should_render(true);
+            self.remove_banner();
+            AdjustedInput::ReRunCommandInThisPane(run_command.clone())
+        })
+    }
+    fn handle_held_drop_to_shell(&mut self) -> Option<AdjustedInput> {
+        self.is_held.take().map(|(_, _, run_command)| {
+            // Drop to shell in the same working directory as the command was run
+            let working_dir = run_command.cwd.clone();
+            self.is_held = None;
+            self.grid.reset_terminal_state();
+            self.set_should_render(true);
+            self.remove_banner();
+            AdjustedInput::DropToShellInThisPane { working_dir }
+        })
+    }
 }
 
 #[cfg(test)]
