@@ -60,6 +60,7 @@ pub struct PluginLoader<'a> {
     plugin_dir: &'a PathBuf,
     tab_index: Option<usize>,
     plugin_own_data_dir: PathBuf,
+    plugin_own_cache_dir: PathBuf,
     size: Size,
     wasm_blob_on_hd: Option<(Vec<u8>, PathBuf)>,
     path_to_default_shell: PathBuf,
@@ -109,7 +110,7 @@ impl<'a> PluginLoader<'a> {
             engine,
             &plugin_dir,
             path_to_default_shell,
-            zellij_cwd,
+            Some(zellij_cwd),
             capabilities,
             client_attributes,
             default_shell,
@@ -275,7 +276,6 @@ impl<'a> PluginLoader<'a> {
         connected_clients: Arc<Mutex<Vec<ClientId>>>,
         loading_indication: &mut LoadingIndication,
         path_to_default_shell: PathBuf,
-        zellij_cwd: PathBuf,
         capabilities: PluginCapabilities,
         client_attributes: ClientAttributes,
         default_shell: Option<TerminalAction>,
@@ -301,7 +301,7 @@ impl<'a> PluginLoader<'a> {
             engine,
             &plugin_dir,
             path_to_default_shell,
-            zellij_cwd,
+            None,
             capabilities,
             client_attributes,
             default_shell,
@@ -345,7 +345,10 @@ impl<'a> PluginLoader<'a> {
         let plugin_own_data_dir = ZELLIJ_SESSION_CACHE_DIR
             .join(Url::from(&plugin.location).to_string())
             .join(format!("{}-{}", plugin_id, client_id));
-        create_plugin_fs_entries(&plugin_own_data_dir)?;
+        let plugin_own_cache_dir = ZELLIJ_SESSION_CACHE_DIR
+            .join(Url::from(&plugin.location).to_string())
+            .join(format!("plugin_cache"));
+        create_plugin_fs_entries(&plugin_own_data_dir, &plugin_own_cache_dir)?;
         let plugin_path = plugin.path.clone();
         Ok(PluginLoader {
             plugin_cache: plugin_cache.clone(),
@@ -359,6 +362,7 @@ impl<'a> PluginLoader<'a> {
             plugin_dir,
             tab_index,
             plugin_own_data_dir,
+            plugin_own_cache_dir,
             size,
             wasm_blob_on_hd: None,
             path_to_default_shell,
@@ -382,7 +386,7 @@ impl<'a> PluginLoader<'a> {
         engine: Engine,
         plugin_dir: &'a PathBuf,
         path_to_default_shell: PathBuf,
-        zellij_cwd: PathBuf,
+        cwd: Option<PathBuf>,
         capabilities: PluginCapabilities,
         client_attributes: ClientAttributes,
         default_shell: Option<TerminalAction>,
@@ -405,6 +409,10 @@ impl<'a> PluginLoader<'a> {
         let keybinds = running_plugin.store.data().keybinds.clone();
         let default_mode = running_plugin.store.data().default_mode;
         let plugin_config = running_plugin.store.data().plugin.clone();
+        // prefer the explicitly given cwd, otherwise copy it from the running plugin
+        // (when reloading a plugin, we want to copy it, when starting a new plugin instance from
+        // meomory, we want to reset it)
+        let zellij_cwd = cwd.unwrap_or_else(|| running_plugin.store.data().plugin_cwd.clone());
         loading_indication.set_name(running_plugin.store.data().name());
         PluginLoader::new(
             plugin_cache,
@@ -786,6 +794,7 @@ impl<'a> PluginLoader<'a> {
         let dirs = vec![
             ("/host".to_owned(), self.zellij_cwd.clone()),
             ("/data".to_owned(), self.plugin_own_data_dir.clone()),
+            ("/cache".to_owned(), self.plugin_own_cache_dir.clone()),
             ("/tmp".to_owned(), ZELLIJ_TMP_DIR.clone()),
         ];
         let dirs = dirs.into_iter().filter(|(_dir_name, dir)| {
@@ -822,6 +831,7 @@ impl<'a> PluginLoader<'a> {
             senders: self.senders.clone(),
             wasi_ctx,
             plugin_own_data_dir: self.plugin_own_data_dir.clone(),
+            plugin_own_cache_dir: self.plugin_own_cache_dir.clone(),
             tab_index: self.tab_index,
             path_to_default_shell: self.path_to_default_shell.clone(),
             capabilities: self.capabilities.clone(),
@@ -859,13 +869,19 @@ impl<'a> PluginLoader<'a> {
     }
 }
 
-fn create_plugin_fs_entries(plugin_own_data_dir: &PathBuf) -> Result<()> {
+fn create_plugin_fs_entries(
+    plugin_own_data_dir: &PathBuf,
+    plugin_own_cache_dir: &PathBuf,
+) -> Result<()> {
     let err_context = || "failed to create plugin fs entries";
     // Create filesystem entries mounted into WASM.
     // We create them here to get expressive error messages in case they fail.
     fs::create_dir_all(&plugin_own_data_dir)
         .with_context(|| format!("failed to create datadir in {plugin_own_data_dir:?}"))
         .with_context(err_context)?;
+    fs::create_dir_all(&plugin_own_cache_dir)
+        .with_context(|| format!("failed to create cache dir in {plugin_own_cache_dir:?}"))
+        .with_context(err_context)?;
     fs::create_dir_all(ZELLIJ_TMP_DIR.as_path())
         .with_context(|| format!("failed to create tmpdir at {:?}", &ZELLIJ_TMP_DIR.as_path()))
         .with_context(err_context)?;
