@@ -1,12 +1,12 @@
 //! Definition of the actions that can be bound to keys.
 
-use super::command::RunCommandAction;
+use super::command::{OpenFilePayload, RunCommandAction};
 use super::layout::{
     FloatingPaneLayout, Layout, PluginAlias, RunPlugin, RunPluginLocation, RunPluginOrAlias,
     SwapFloatingLayout, SwapTiledLayout, TiledPaneLayout,
 };
 use crate::cli::CliAction;
-use crate::data::{Direction, Resize};
+use crate::data::{Direction, KeyWithModifier, Resize};
 use crate::data::{FloatingPaneCoordinates, InputMode};
 use crate::home::{find_default_config_dir, get_layout_dir};
 use crate::input::config::{Config, ConfigError, KdlError};
@@ -102,7 +102,7 @@ pub enum Action {
     /// Quit Zellij.
     Quit,
     /// Write to the terminal.
-    Write(Vec<u8>),
+    Write(Option<KeyWithModifier>, Vec<u8>, bool), // bool -> is_kitty_keyboard_protocol
     /// Write Characters to the terminal.
     WriteChars(String),
     /// Switch to the specified input mode.
@@ -157,17 +157,17 @@ pub enum Action {
     ToggleActiveSyncTab,
     /// Open a new pane in the specified direction (relative to focus).
     /// If no direction is specified, will try to use the biggest available space.
-    NewPane(Option<Direction>, Option<String>), // String is an optional pane name
-    /// Open the file in a new pane using the default editor
+    NewPane(Option<Direction>, Option<String>, bool), // String is an optional pane name
+    /// Open the file in a new pane using the default editor, bool -> start suppressed
     EditFile(
-        PathBuf,
-        Option<usize>,
-        Option<PathBuf>,
+        OpenFilePayload,
         Option<Direction>,
         bool,
         bool,
+        bool,
         Option<FloatingPaneCoordinates>,
-    ), // usize is an optional line number, Option<PathBuf> is an optional cwd, bool is floating true/false, second bool is in_place
+    ), // bool is floating true/false, second bool is in_place
+    // third bool is start_suppressed
     /// Open a new floating pane
     NewFloatingPane(
         Option<RunCommandAction>,
@@ -195,6 +195,7 @@ pub enum Action {
         Option<Vec<SwapTiledLayout>>,
         Option<Vec<SwapFloatingLayout>>,
         Option<String>,
+        bool, // should_change_focus_to_new_tab
     ), // the String is the tab name
     /// Do nothing.
     NoOp,
@@ -289,6 +290,7 @@ pub enum Action {
         payload: Option<String>,
         args: Option<BTreeMap<String, String>>,
         plugin: Option<String>,
+        plugin_id: Option<u32>, // supercedes plugin if present
         configuration: Option<BTreeMap<String, String>>,
         launch_new: bool,
         skip_cache: bool,
@@ -317,7 +319,7 @@ impl Action {
         config: Option<Config>,
     ) -> Result<Vec<Action>, String> {
         match cli_action {
-            CliAction::Write { bytes } => Ok(vec![Action::Write(bytes)]),
+            CliAction::Write { bytes } => Ok(vec![Action::Write(None, bytes, false)]),
             CliAction::WriteChars { chars } => Ok(vec![Action::WriteChars(chars)]),
             CliAction::Resize { resize, direction } => Ok(vec![Action::Resize(resize, direction)]),
             CliAction::FocusNextPane => Ok(vec![Action::FocusNextPane]),
@@ -432,6 +434,7 @@ impl Action {
                         direction,
                         hold_on_close,
                         hold_on_start,
+                        ..Default::default()
                     };
                     if floating {
                         Ok(vec![Action::NewFloatingPane(
@@ -484,13 +487,13 @@ impl Action {
                         file = cwd.join(file);
                     }
                 }
+                let start_suppressed = false;
                 Ok(vec![Action::EditFile(
-                    file,
-                    line_number,
-                    cwd,
+                    OpenFilePayload::new(file, line_number, cwd),
                     direction,
                     floating,
                     in_place,
+                    start_suppressed,
                     FloatingPaneCoordinates::new(x, y, width, height),
                 )])
             },
@@ -529,10 +532,28 @@ impl Action {
                     let layout_dir = layout_dir
                         .or_else(|| config.and_then(|c| c.options.layout_dir))
                         .or_else(|| get_layout_dir(find_default_config_dir()));
-                    let (path_to_raw_layout, raw_layout, swap_layouts) =
+
+                    let mut should_start_layout_commands_suspended = false;
+                    let (path_to_raw_layout, raw_layout, swap_layouts) = if let Some(layout_url) =
+                        layout_path.to_str().and_then(|l| {
+                            if l.starts_with("http://") || l.starts_with("https://") {
+                                Some(l)
+                            } else {
+                                None
+                            }
+                        }) {
+                        should_start_layout_commands_suspended = true;
+                        (
+                            layout_url.to_owned(),
+                            Layout::stringified_from_url(layout_url)
+                                .map_err(|e| format!("Failed to load layout: {}", e))?,
+                            None,
+                        )
+                    } else {
                         Layout::stringified_from_path_or_default(Some(&layout_path), layout_dir)
-                            .map_err(|e| format!("Failed to load layout: {}", e))?;
-                    let layout = Layout::from_str(&raw_layout, path_to_raw_layout, swap_layouts.as_ref().map(|(f, p)| (f.as_str(), p.as_str())), cwd).map_err(|e| {
+                            .map_err(|e| format!("Failed to load layout: {}", e))?
+                    };
+                    let mut layout = Layout::from_str(&raw_layout, path_to_raw_layout, swap_layouts.as_ref().map(|(f, p)| (f.as_str(), p.as_str())), cwd).map_err(|e| {
                         let stringified_error = match e {
                             ConfigError::KdlError(kdl_error) => {
                                 let error = kdl_error.add_src(layout_path.as_path().as_os_str().to_string_lossy().to_string(), String::from(raw_layout));
@@ -564,36 +585,62 @@ impl Action {
                         };
                         stringified_error
                     })?;
+                    if should_start_layout_commands_suspended {
+                        layout.recursively_add_start_suspended_including_template(Some(true));
+                    }
                     let mut tabs = layout.tabs();
-                    if tabs.len() > 1 {
-                        return Err(format!("Tab layout cannot itself have tabs"));
-                    } else if !tabs.is_empty() {
+                    if !tabs.is_empty() {
                         let swap_tiled_layouts = Some(layout.swap_tiled_layouts.clone());
                         let swap_floating_layouts = Some(layout.swap_floating_layouts.clone());
-                        let (tab_name, layout, floating_panes_layout) =
-                            tabs.drain(..).next().unwrap();
-                        let name = tab_name.or(name);
-                        Ok(vec![Action::NewTab(
-                            Some(layout),
-                            floating_panes_layout,
-                            swap_tiled_layouts,
-                            swap_floating_layouts,
-                            name,
-                        )])
+                        let mut new_tab_actions = vec![];
+                        let mut has_focused_tab = tabs
+                            .iter()
+                            .any(|(_, layout, _)| layout.focus.unwrap_or(false));
+                        for (tab_name, layout, floating_panes_layout) in tabs.drain(..) {
+                            let name = tab_name.or_else(|| name.clone());
+                            let should_change_focus_to_new_tab =
+                                layout.focus.unwrap_or_else(|| {
+                                    if !has_focused_tab {
+                                        has_focused_tab = true;
+                                        true
+                                    } else {
+                                        false
+                                    }
+                                });
+                            new_tab_actions.push(Action::NewTab(
+                                Some(layout),
+                                floating_panes_layout,
+                                swap_tiled_layouts.clone(),
+                                swap_floating_layouts.clone(),
+                                name,
+                                should_change_focus_to_new_tab,
+                            ));
+                        }
+                        Ok(new_tab_actions)
                     } else {
                         let swap_tiled_layouts = Some(layout.swap_tiled_layouts.clone());
                         let swap_floating_layouts = Some(layout.swap_floating_layouts.clone());
                         let (layout, floating_panes_layout) = layout.new_tab();
+                        let should_change_focus_to_new_tab = true;
                         Ok(vec![Action::NewTab(
                             Some(layout),
                             floating_panes_layout,
                             swap_tiled_layouts,
                             swap_floating_layouts,
                             name,
+                            should_change_focus_to_new_tab,
                         )])
                     }
                 } else {
-                    Ok(vec![Action::NewTab(None, vec![], None, None, name)])
+                    let should_change_focus_to_new_tab = true;
+                    Ok(vec![Action::NewTab(
+                        None,
+                        vec![],
+                        None,
+                        None,
+                        name,
+                        should_change_focus_to_new_tab,
+                    )])
                 }
             },
             CliAction::PreviousSwapLayout => Ok(vec![Action::PreviousSwapLayout]),
@@ -693,6 +740,17 @@ impl Action {
             CliAction::ListClients => Ok(vec![Action::ListClients]),
         }
     }
+    pub fn launches_plugin(&self, plugin_url: &str) -> bool {
+        match self {
+            Action::LaunchPlugin(run_plugin_or_alias, ..) => {
+                &run_plugin_or_alias.location_string() == plugin_url
+            },
+            Action::LaunchOrFocusPlugin(run_plugin_or_alias, ..) => {
+                &run_plugin_or_alias.location_string() == plugin_url
+            },
+            _ => false,
+        }
+    }
 }
 
 impl From<OnForceClose> for Action {
