@@ -17,7 +17,7 @@ use crate::data::{Direction, InputMode, ResizeStrategy};
 use crate::errors::prelude::*;
 use crate::input::actions::Action;
 use crate::input::actions::{SearchDirection, SearchOption};
-use crate::input::command::RunCommandAction;
+use crate::input::command::{OpenFilePayload, RunCommandAction};
 use crate::input::layout::{
     PluginUserConfiguration, RunPlugin, RunPluginLocation, RunPluginOrAlias,
 };
@@ -37,7 +37,7 @@ impl TryFrom<ProtobufAction> for Action {
             },
             Some(ProtobufActionName::Write) => match protobuf_action.optional_payload {
                 Some(OptionalPayload::WritePayload(write_payload)) => {
-                    Ok(Action::Write(write_payload.bytes_to_write))
+                    Ok(Action::Write(None, write_payload.bytes_to_write, false))
                 },
                 _ => Err("Wrong payload for Action::Write"),
             },
@@ -220,7 +220,7 @@ impl TryFrom<ProtobufAction> for Action {
                         .and_then(|d| ProtobufResizeDirection::from_i32(d))
                         .and_then(|d| d.try_into().ok());
                     let pane_name = payload.pane_name;
-                    Ok(Action::NewPane(direction, pane_name))
+                    Ok(Action::NewPane(direction, pane_name, false))
                 },
                 _ => Err("Wrong payload for Action::NewPane"),
             },
@@ -236,12 +236,11 @@ impl TryFrom<ProtobufAction> for Action {
                     let should_float = payload.should_float;
                     let should_be_in_place = false;
                     Ok(Action::EditFile(
-                        file_to_edit,
-                        line_number,
-                        cwd,
+                        OpenFilePayload::new(file_to_edit, line_number, cwd),
                         direction,
                         should_float,
                         should_be_in_place,
+                        false,
                         None,
                     ))
                 },
@@ -314,7 +313,7 @@ impl TryFrom<ProtobufAction> for Action {
                     Some(_) => Err("NewTab should not have a payload"),
                     None => {
                         // we do not serialize the layouts of this action
-                        Ok(Action::NewTab(None, vec![], None, None, None))
+                        Ok(Action::NewTab(None, vec![], None, None, None, true))
                     },
                 }
             },
@@ -704,6 +703,7 @@ impl TryFrom<ProtobufAction> for Action {
                     in_place: None,
                     cwd: None,
                     pane_title: None,
+                    plugin_id: None,
                 }),
             },
             _ => Err("Unknown Action"),
@@ -719,7 +719,7 @@ impl TryFrom<Action> for ProtobufAction {
                 name: ProtobufActionName::Quit as i32,
                 optional_payload: None,
             }),
-            Action::Write(bytes) => Ok(ProtobufAction {
+            Action::Write(_, bytes, _) => Ok(ProtobufAction {
                 name: ProtobufActionName::Write as i32,
                 optional_payload: Some(OptionalPayload::WritePayload(WritePayload {
                     bytes_to_write: bytes,
@@ -885,7 +885,7 @@ impl TryFrom<Action> for ProtobufAction {
                 name: ProtobufActionName::ToggleActiveSyncTab as i32,
                 optional_payload: None,
             }),
-            Action::NewPane(direction, new_pane_name) => {
+            Action::NewPane(direction, new_pane_name, _start_suppressed) => {
                 let direction = direction.and_then(|direction| {
                     let protobuf_direction: ProtobufResizeDirection = direction.try_into().ok()?;
                     Some(protobuf_direction as i32)
@@ -899,20 +899,19 @@ impl TryFrom<Action> for ProtobufAction {
                 })
             },
             Action::EditFile(
-                path_to_file,
-                line_number,
-                cwd,
+                open_file_payload,
                 direction,
                 should_float,
                 _should_be_in_place,
                 _floating_pane_coordinates,
+                _start_suppressed,
             ) => {
-                let file_to_edit = path_to_file.display().to_string();
-                let cwd = cwd.map(|cwd| cwd.display().to_string());
+                let file_to_edit = open_file_payload.path.display().to_string();
+                let cwd = open_file_payload.cwd.map(|cwd| cwd.display().to_string());
                 let direction: Option<i32> = direction
                     .and_then(|d| ProtobufResizeDirection::try_from(d).ok())
                     .map(|d| d as i32);
-                let line_number = line_number.map(|l| l as u32);
+                let line_number = open_file_payload.line_number.map(|l| l as u32);
                 Ok(ProtobufAction {
                     name: ProtobufActionName::EditFile as i32,
                     optional_payload: Some(OptionalPayload::EditFilePayload(EditFilePayload {
@@ -1385,6 +1384,7 @@ impl TryFrom<ProtobufRunCommandAction> for RunCommandAction {
             direction,
             hold_on_close,
             hold_on_start,
+            ..Default::default()
         })
     }
 }
