@@ -4,6 +4,7 @@ pub use super::generated_api::api::plugin_ids::{
 use crate::data::PluginIds;
 
 use std::convert::TryFrom;
+use std::path::PathBuf;
 
 impl TryFrom<ProtobufPluginIds> for PluginIds {
     type Error = &'static str;
@@ -11,6 +12,7 @@ impl TryFrom<ProtobufPluginIds> for PluginIds {
         Ok(PluginIds {
             plugin_id: protobuf_plugin_ids.plugin_id as u32,
             zellij_pid: protobuf_plugin_ids.zellij_pid as u32,
+            initial_cwd: PathBuf::from(protobuf_plugin_ids.initial_cwd),
         })
     }
 }
@@ -21,6 +23,7 @@ impl TryFrom<PluginIds> for ProtobufPluginIds {
         Ok(ProtobufPluginIds {
             plugin_id: plugin_ids.plugin_id as i32,
             zellij_pid: plugin_ids.zellij_pid as i32,
+            initial_cwd: plugin_ids.initial_cwd.display().to_string(),
         })
     }
 }
